@@ -1,22 +1,19 @@
 /*******************************************************************************
   Copyright(c) 2010 Gerry Rozema. All rights reserved.
 
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the Free
-  Software Foundation; either version 2 of the License, or (at your option)
-  any later version.
-
-  This program is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-  more details.
-
-  You should have received a copy of the GNU General Public License along with
-  this program; if not, write to the Free Software Foundation, Inc., 59
-  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-
-  The full GNU General Public License is included in this distribution in the
-  file called LICENSE.
+ This library is free software; you can redistribute it and/or
+ modify it under the terms of the GNU Library General Public
+ License version 2 as published by the Free Software Foundation.
+ .
+ This library is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ Library General Public License for more details.
+ .
+ You should have received a copy of the GNU Library General Public License
+ along with this library; see the file COPYING.LIB.  If not, write to
+ the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ Boston, MA 02110-1301, USA.
 *******************************************************************************/
 #include "ccd_simulator.h"
 
@@ -83,7 +80,8 @@ void ISNewBLOB (const char *dev, const char *name, int sizes[], int blobsizes[],
 }
 void ISSnoopDevice (XMLEle *root)
 {
-    INDI_UNUSED(root);
+    ISInit();
+    ccdsim->ISSnoopDevice(root);
 }
 
 
@@ -124,10 +122,14 @@ CCDSim::CCDSim()
     GuideRate=7;    //  guide rate is 7 arcseconds per second
     TimeFactor=1;
 
+    SimulatorSettingsNV = new INumberVectorProperty;
+    TimeFactorSV = new ISwitchVectorProperty;
+
 }
 
 bool CCDSim::SetupParms()
 {
+    int nbuf;
     SetCCDParams(SimulatorSettingsN[0].value,SimulatorSettingsN[1].value,16,SimulatorSettingsN[2].value,SimulatorSettingsN[3].value);
     //  Kwiq
     maxnoise=SimulatorSettingsN[10].value;
@@ -140,12 +142,9 @@ bool CCDSim::SetupParms()
     OAGoffset=SimulatorSettingsN[12].value;    //  An oag is offset this much from center of scope position (arcminutes);
     seeing=SimulatorSettingsN[9].value;        //  we get real fat stars in this one
 
-
-    if(RawFrame != NULL) delete RawFrame;
-    RawFrameSize=XRes*YRes;                 //  this is pixel count
-    RawFrameSize=RawFrameSize*2;            //  Each pixel is 2 bytes
-    RawFrameSize+=512;                      //  leave a little extra at the end
-    RawFrame=new char[RawFrameSize];
+    nbuf = PrimaryCCD.getXRes() * PrimaryCCD.getYRes() * PrimaryCCD.getBPP();
+    nbuf += 512;
+    PrimaryCCD.setFrameBufferSize(nbuf);
 
     return true;
 }
@@ -153,13 +152,14 @@ bool CCDSim::SetupParms()
 bool CCDSim::Connect()
 {
 
+    int nbuf;
     SetupParms();
 
-    if(HasGuideHead) {
+    if(HasGuideHead)
+    {
         SetGuidHeadParams(500,290,8,9.8,12.6);
-        RawGuideSize=GXRes*GYRes;                 //  this is pixel count
-        RawGuideSize+=512;                      //  leave a little extra at the end
-        RawGuiderFrame=new char[RawGuideSize];
+        nbuf = GuideCCD.getXRes() * GuideCCD.getYRes();
+        GuideCCD.setFrameBufferSize(nbuf);
     }
 
     SetTimer(1000);     //  start the timer
@@ -173,14 +173,7 @@ CCDSim::~CCDSim()
 
 const char * CCDSim::getDefaultName()
 {
-    //fprintf(stderr,"Arrived in getDefaultName and deviceName returns '%s'\n",deviceName());
-    //if(strlen(deviceName())==0) {
         return (char *)"CCD Simulator";
-    //} else {
-    //    char n[500];
-    //    strcpy(n,deviceName());
-    //    return n;
-    //}
 }
 
 bool CCDSim::initProperties()
@@ -189,14 +182,8 @@ bool CCDSim::initProperties()
     //  but the simulators are a special case
     INDI::CCD::initProperties();
 
-    IUFillNumber(&EqN[0],"RA","Ra (hh:mm:ss)","%010.6m",0,24,0,0);
-    IUFillNumber(&EqN[1],"DEC","Dec (dd:mm:ss)","%010.6m",-90,90,0,0);
-    IUFillNumberVector(&EqNV,EqN,2,"","EQUATORIAL_EOD_COORD","Eq. Coordinates","Main Control",IP_RW,60,IPS_IDLE);
-    //IUFillNumberVector(&EqNV,EqN,2,deviceName(),"EQUATORIAL_EOD_COORD","Eq. Coordinates","Main Control",IP_RW,60,IPS_IDLE);
-
-    SimulatorSettingsNV = new INumberVectorProperty;
-    IUFillNumber(&SimulatorSettingsN[0],"SIM_XRES","CCD X resolution","%4.0f",0,2048,0,1280);
-    IUFillNumber(&SimulatorSettingsN[1],"SIM_YRES","CCD Y resolution","%4.0f",0,2048,0,1024);
+    IUFillNumber(&SimulatorSettingsN[0],"SIM_PrimarCCD.getXRes()","CCD X resolution","%4.0f",0,2048,0,1280);
+    IUFillNumber(&SimulatorSettingsN[1],"SIM_PrimarCCD.getYRes()","CCD Y resolution","%4.0f",0,2048,0,1024);
     IUFillNumber(&SimulatorSettingsN[2],"SIM_XSIZE","CCD X Pixel Size","%4.2f",0,60,0,5.2);
     IUFillNumber(&SimulatorSettingsN[3],"SIM_YSIZE","CCD Y Pixel Size","%4.2f",0,60,0,5.2);
     IUFillNumber(&SimulatorSettingsN[4],"SIM_MAXVAL","CCD Maximum ADU","%4.0f",0,65000,0,65000);
@@ -208,22 +195,16 @@ bool CCDSim::initProperties()
     IUFillNumber(&SimulatorSettingsN[10],"SIM_NOISE","CCD Noise","%4.0f",0,6000,0,50);
     IUFillNumber(&SimulatorSettingsN[11],"SIM_SKYGLOW","Sky Glow (magnitudes)","%4.1f",0,6000,0,19.5);
     IUFillNumber(&SimulatorSettingsN[12],"SIM_OAGOFFSET","Oag Offset (arminutes)","%4.1f",0,6000,0,0);
-    IUFillNumberVector(SimulatorSettingsNV,SimulatorSettingsN,13,deviceName(),"SIMULATOR_SETTINGS","Simulator Settings","SimSettings",IP_RW,60,IPS_IDLE);
+    IUFillNumberVector(SimulatorSettingsNV,SimulatorSettingsN,13,deviceName(),"SIMULATOR_SETTINGS","Simulator Settings","Simulator Config",IP_RW,60,IPS_IDLE);
 
 
-    TelescopeTV = new ITextVectorProperty;
-    IUFillText(&TelescopeT[0],"ACTIVE_TELESCOPE","Telescope","");
-    IUFillTextVector(TelescopeTV,TelescopeT,1,deviceName(),"ACTIVE_DEVICES","Snoop Scope","Simulator Config",IP_RW,60,IPS_IDLE);
-
-    TimeFactorSV = new ISwitchVectorProperty;
-
     IUFillSwitch(&TimeFactorS[0],"1X","Actual Time",ISS_ON);
     IUFillSwitch(&TimeFactorS[1],"10X","10x",ISS_OFF);
     IUFillSwitch(&TimeFactorS[2],"100X","100x",ISS_OFF);
     IUFillSwitchVector(TimeFactorSV,TimeFactorS,3,deviceName(),"ON_TIME_FACTOR","Time Factor","Simulator Config",IP_RW,ISR_1OFMANY,60,IPS_IDLE);
 
 
-    loadConfig();
+    //loadConfig();
 
     return true;
 }
@@ -237,7 +218,6 @@ void CCDSim::ISGetProperties (const char *dev)
 
     defineNumber(SimulatorSettingsNV);
     defineSwitch(TimeFactorSV);
-    defineText(TelescopeTV);
     //IDDefText(&ConfigFileTV, NULL);
     //IDDefSwitch(&ConfigSaveRestoreSV, NULL);
 
@@ -256,16 +236,6 @@ bool CCDSim::updateProperties()
 
 bool CCDSim::Disconnect()
 {
-    delete RawFrame;
-    RawFrameSize=0;
-    RawFrame=NULL;
-
-
-    if(RawGuiderFrame != NULL) {
-        delete RawGuiderFrame;
-        RawGuideSize=0;
-    }
-
     return true;
 }
 
@@ -334,6 +304,14 @@ void CCDSim::TimerHit()
     {
         float timeleft;
         timeleft=CalcTimeLeft(ExpStart,ExposureRequest);
+
+        if (timeleft < 0)
+             timeleft = 0;
+
+        PrimaryCCD.setExposure(timeleft);
+        //ImageExposureN[0].value = timeleft;
+        //IDSetNumber(ImageExposureNP, NULL);
+
         if(timeleft < 1.0)
         {
             if(timeleft <= 0.001)
@@ -346,10 +324,20 @@ void CCDSim::TimerHit()
             }
         }
     }
+
     if(InGuideExposure)
     {
         float timeleft;
         timeleft=CalcTimeLeft(GuideExpStart,GuideExposureRequest);
+
+
+        if (timeleft < 0)
+             timeleft = 0;
+
+        //ImageExposureN[0].value = timeleft;
+        //IDSetNumber(ImageExposureNP, NULL);
+        GuideCCD.setExposure(timeleft);
+
         if(timeleft < 1.0)
         {
             if(timeleft <= 0.001)
@@ -389,9 +377,10 @@ int CCDSim::DrawCcdFrame()
     unsigned short int *ptr;
     unsigned short int val;
 
-    ptr=(unsigned short int *)RawFrame;
+    ptr=(unsigned short int *) PrimaryCCD.getFrameBuffer();
 
-    if(ShowStarField) {
+    if(ShowStarField)
+    {
         char gsccmd[250];
         FILE *pp;
         int stars=0;
@@ -422,7 +411,7 @@ int CCDSim::DrawCcdFrame()
         //PeOffset=PeOffset/15;       //  ra is in h:mm
 
         //  Start by clearing the frame buffer
-        memset(RawFrame,0,RawFrameSize);
+        memset(PrimaryCCD.getFrameBuffer(),0,PrimaryCCD.getFrameBufferSize());
 
 
         //  Spin up a set of plate constants that will relate
@@ -438,14 +427,14 @@ int CCDSim::DrawCcdFrame()
         //  for now we cheat
         //  no offset or rotation for and y axis means
         pb=0.0;
-        pc=XRes/2/BinX;
+        pc=PrimaryCCD.getXRes()/2/PrimaryCCD.getBinX();
         pd=0.0;
-        pf=YRes/2/BinY;
+        pf=PrimaryCCD.getYRes()/2/PrimaryCCD.getBinY();
         //  and we do a simple scale for x and y locations
         //  based on the focal length and pixel size
         //  focal length in mm, pixels in microns
-        pa=focallength/PixelSizex*1000/BinX;
-        pe=focallength/PixelSizey*1000/BinY;
+        pa=focallength/PrimaryCCD.getPixelSizeX()*1000/PrimaryCCD.getBinX();
+        pe=focallength/PrimaryCCD.getPixelSizeY()*1000/PrimaryCCD.getBinY();
 
         //IDLog("Pixels are %4.2f %4.2f  pa %6.4f  pe %6.4f\n",PixelSizex,PixelSizey,pa,pe);
 
@@ -477,7 +466,7 @@ int CCDSim::DrawCcdFrame()
         //  now lets calculate the radius we need to fetch
         float radius;
 
-        radius=sqrt((Scalex*Scalex*XRes/2.0*XRes/2.0)+(Scaley*Scaley*YRes/2.0*YRes/2.0));
+        radius=sqrt((Scalex*Scalex*PrimaryCCD.getXRes()/2.0*PrimaryCCD.getXRes()/2.0)+(Scaley*Scaley*PrimaryCCD.getYRes()/2.0*PrimaryCCD.getYRes()/2.0));
         //  we have radius in arcseconds now
         radius=radius/60;   //  convert to arcminutes
         //fprintf(stderr,"Lookup radius %4.2f\n",radius);
@@ -504,7 +493,7 @@ int CCDSim::DrawCcdFrame()
         if(radius > 60) lookuplimit=11;
 
         //  if this is a light frame, we need a star field drawn
-        if(FrameType==FRAME_TYPE_LIGHT)
+        if(PrimaryCCD.getFrameType()==CCDChip::LIGHT_FRAME)
         {
             //sprintf(gsccmd,"gsc -c %8.6f %+8.6f -r 120 -m 0 9.1",rad+PEOffset,Dec);
             sprintf(gsccmd,"gsc -c %8.6f %+8.6f -r %4.1f -m 0 %4.2f -n 3000",rad+PEOffset,cameradec,radius,lookuplimit);
@@ -592,13 +581,15 @@ int CCDSim::DrawCcdFrame()
         //  this is essentially the same math as drawing a dim star with
         //  fwhm equivalent to the full field of view
 
-        if((FrameType==FRAME_TYPE_LIGHT)||(FrameType==FRAME_TYPE_FLAT))
+        CCDChip::CCD_FRAME ftype = PrimaryCCD.getFrameType();
+
+        if((ftype==CCDChip::LIGHT_FRAME)||(ftype==CCDChip::FLAT_FRAME))
         {
             float skyflux;
             float glow;
             //  calculate flux from our zero point and gain values
             glow=skyglow;
-            if(FrameType==FRAME_TYPE_FLAT)
+            if(ftype==CCDChip::FLAT_FRAME)
             {
                 //  Assume flats are done with a diffuser
                 //  in broad daylight, so, the sky magnitude
@@ -609,24 +600,27 @@ int CCDSim::DrawCcdFrame()
             skyflux=pow(10,((glow-z)*k/-2.5));
             //  ok, flux represents one second now
             //  scale up linearly for exposure time
-            skyflux=skyflux*ExposureRequest*BinX*BinY;
+            skyflux=skyflux*ExposureRequest*PrimaryCCD.getBinX()*PrimaryCCD.getBinY();
             //IDLog("SkyFlux = %4.2f ExposureRequest %4.2f\n",skyflux,ExposureRequest);
 
             unsigned short *pt;
-            pt=(unsigned short int *)RawFrame;
-            for(int y=0; y<YRes/BinY; y++) {
-                for(int x=0; x<XRes/BinX; x++) {
+
+            int nwidth  = PrimaryCCD.getXRes()/PrimaryCCD.getBinX();
+            int nheight = PrimaryCCD.getYRes()/PrimaryCCD.getBinY();
+            pt=(unsigned short int *)PrimaryCCD.getFrameBuffer();
+            for(int y=0; y< nheight; y++) {
+                for(int x=0; x< nwidth; x++) {
                     float dc;   //  distance from center
                     float fp;   //  flux this pixel;
                     float sx,sy;
                     float vig;
 
-                    sx=XRes/2/BinX;
+                    sx=PrimaryCCD.getXRes()/2/PrimaryCCD.getBinX();
                     sx=sx-x;
-                    sy=YRes/2/BinY;
+                    sy=PrimaryCCD.getYRes()/2/PrimaryCCD.getBinY();
                     sy=sy-y;
 
-                    vig=XRes/BinX;
+                    vig=PrimaryCCD.getXRes()/PrimaryCCD.getBinX();
                     vig=vig*ImageScalex;
                     //  need to make this account for actual pixel size
                     dc=sqrt(sx*sx*ImageScalex*ImageScalex+sy*sy*ImageScaley*ImageScaley);
@@ -653,8 +647,8 @@ int CCDSim::DrawCcdFrame()
 
 
         //  Now we add some bias and read noise
-        for(x=0; x<XRes; x++) {
-            for(y=0; y<YRes; y++) {
+        for(x=0; x<PrimaryCCD.getXRes(); x++) {
+            for(y=0; y<PrimaryCCD.getYRes(); y++) {
                 int noise;
 
                 noise=random();
@@ -670,7 +664,9 @@ int CCDSim::DrawCcdFrame()
         if(testvalue > 255) testvalue=0;
         val=testvalue;
 
-        for(int x=0; x<XRes*YRes; x++)
+        int nbuf    = PrimaryCCD.getXRes()*PrimaryCCD.getYRes();
+
+        for(int x=0; x<nbuf; x++)
         {
             *ptr=val++;
             ptr++;
@@ -685,12 +681,14 @@ int CCDSim::DrawGuiderFrame()
     unsigned char *ptr;
     unsigned char val;
 
-    ptr=(unsigned char *)RawGuiderFrame;
+    ptr=(unsigned char *) GuideCCD.getFrameBuffer();
     testvalue++;
     if(testvalue > 255) testvalue=0;
     val=testvalue;
 
-    for(int x=0; x<GXRes*GYRes; x++) {
+    int nbuf = GuideCCD.getXRes()*GuideCCD.getYRes();
+    for(int x=0; x< nbuf; x++)
+    {
         *ptr=val++;
         ptr++;
     }
@@ -707,7 +705,7 @@ int CCDSim::DrawImageStar(float mag,float x,float y)
     int boxsizey=5;
     float flux;
 
-    if((x<0)||(x>XRes/BinX)||(y<0)||(y>YRes/BinY))
+    if((x<0)||(x>PrimaryCCD.getXRes()/PrimaryCCD.getBinX())||(y<0)||(y>PrimaryCCD.getYRes()/PrimaryCCD.getBinY()))
     {
         //  this star is not on the ccd frame anyways
         return 0;
@@ -749,7 +747,7 @@ int CCDSim::DrawImageStar(float mag,float x,float y)
             //  a simple linear function
             float fa;
             fa=exp(-2.0*0.7*(dc*dc)/seeing/seeing);
-            fp=fa*flux*BinX*BinY;
+            fp=fa*flux*PrimaryCCD.getBinX()*PrimaryCCD.getBinY();
             if(fp < 0) fp=0;
 
             /*
@@ -772,14 +770,14 @@ int CCDSim::AddToPixel(int x,int y,int val)
 {
     int drew=0;
     if(x >= 0) {
-        if(x < XRes/BinX) {
+        if(x < PrimaryCCD.getXRes()/PrimaryCCD.getBinX()) {
             if(y >= 0) {
-                if(y < YRes/BinY) {
+                if(y < PrimaryCCD.getYRes()/PrimaryCCD.getBinY()) {
                     unsigned short *pt;
                     int newval;
                     drew++;
-                    pt=(unsigned short int *)RawFrame;
-                    pt+=(y*XRes/BinX);
+                    pt=(unsigned short int *)PrimaryCCD.getFrameBuffer();
+                    pt+=(y*PrimaryCCD.getXRes()/PrimaryCCD.getBinX());
                     pt+=x;
                     newval=pt[0];
                     newval+=val;
@@ -792,7 +790,7 @@ int CCDSim::AddToPixel(int x,int y,int val)
     return drew;
 }
 
-int CCDSim::GuideNorth(float v)
+bool CCDSim::GuideNorth(float v)
 {
     float c;
 
@@ -800,9 +798,9 @@ int CCDSim::GuideNorth(float v)
     c=c/3600;
     Dec=Dec+c;
 
-    return 0;
+    return true;
 }
-int CCDSim::GuideSouth(float v)
+bool CCDSim::GuideSouth(float v)
 {
     float c;
 
@@ -810,10 +808,10 @@ int CCDSim::GuideSouth(float v)
     c=c/3600;
     Dec=Dec-c;
 
-    return 0;
+    return true;
 }
 
-int CCDSim::GuideEast(float v)
+bool CCDSim::GuideEast(float v)
 {
     float c;
 
@@ -822,9 +820,9 @@ int CCDSim::GuideEast(float v)
     c=c/(cos(Dec*0.0174532925));
     RA=RA-c;
 
-    return 0;
+    return true;
 }
-int CCDSim::GuideWest(float v)
+bool CCDSim::GuideWest(float v)
 {
     float c;
 
@@ -833,32 +831,15 @@ int CCDSim::GuideWest(float v)
     c=c/(cos(Dec*0.0174532925));
     RA=RA+c;
 
-    return 0;
+    return true;
 }
 
-void CCDSim::ISSnoopDevice (XMLEle *root)
- {
-     //fprintf(stderr,"CCDSim handling snoop\n");
-     if(IUSnoopNumber(root,&EqNV)==0) {
-        float newra,newdec;
-        newra=EqN[0].value;
-        newdec=EqN[1].value;
-        if((newra != RA)||(newdec != Dec)) {
-            //fprintf(stderr,"RA %4.2f  Dec %4.2f Snooped RA %4.2f  Dec %4.2f\n",RA,Dec,newra,newdec);
-            RA=newra;
-            Dec=newdec;
-
-        }
-     } else {
-        //fprintf(stderr,"Snoop Failed\n");
-     }
- }
-
 bool CCDSim::ISNewNumber (const char *dev, const char *name, double values[], char *names[], int n)
 {
     //  first check if it's for our device
     //IDLog("INDI::CCD::ISNewNumber %s\n",name);
-    if(strcmp(dev,deviceName())==0) {
+    if(strcmp(dev,deviceName())==0)
+    {
         //  This is for our device
         //  Now lets see if it's something we process here
 
@@ -898,33 +879,6 @@ bool CCDSim::ISNewNumber (const char *dev, const char *name, double values[], ch
     //  give it a shot
     return INDI::CCD::ISNewNumber(dev,name,values,names,n);
 }
-bool CCDSim::ISNewText (const char *dev, const char *name, char *texts[], char *names[], int n)
-{
-    //  Ok, lets see if this is a property wer process
-    //IDLog("IndiTelescope got %d new text items name %s\n",n,name);
-    //  first check if it's for our device
-    if(strcmp(dev,deviceName())==0) {
-        //  This is for our device
-        //  Now lets see if it's something we process here
-        if(strcmp(name,TelescopeTV->name)==0) {
-            int rc;
-            //IDLog("calling update text\n");
-            TelescopeTV->s=IPS_OK;
-            rc=IUUpdateText(TelescopeTV,texts,names,n);
-            //IDLog("update text returns %d\n",rc);
-            //  Update client display
-            IDSetText(TelescopeTV,NULL);
-            saveConfig();
-            IUFillNumberVector(&EqNV,EqN,2,TelescopeT[0].text,"EQUATORIAL_EOD_COORD","Eq. Coordinates","Main Control",IP_RW,60,IPS_IDLE);
-            IDSnoopDevice(TelescopeT[0].text,"EQUATORIAL_EOD_COORD");
-            //  We processed this one, so, tell the world we did it
-            return true;
-        }
-
-    }
-
-    return INDI::DefaultDriver::ISNewText(dev,name,texts,names,n);
-}
 
 
 bool CCDSim::ISNewSwitch (const char *dev, const char *name, ISState *states, char *names[], int n)
