@@ -1,3 +1,21 @@
+/*******************************************************************************
+  Copyright(c) 2011 Jasem Mutlaq. All rights reserved.
+
+ This library is free software; you can redistribute it and/or
+ modify it under the terms of the GNU Library General Public
+ License version 2 as published by the Free Software Foundation.
+
+ This library is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ Library General Public License for more details.
+
+ You should have received a copy of the GNU Library General Public License
+ along with this library; see the file COPYING.LIB.  If not, write to
+ the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ Boston, MA 02110-1301, USA.
+*******************************************************************************/
+
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -8,15 +26,79 @@
 #include "indicom.h"
 #include "base64.h"
 
+PropertyContainer::PropertyContainer()
+{
+    pPtr = NULL;
+    pRegistered = false;
+    pDynamic = false;
+    pType = INDI_UNKNOWN;
+}
+
+PropertyContainer::~PropertyContainer()
+{
+    // Only delete properties if they were created dynamically via the buildSkeleton
+    // function. Other drivers are responsible for their own memory allocation.
+    if (pDynamic)
+    {
+        switch (pType)
+        {
+        case INDI_NUMBER:
+         delete ((INumberVectorProperty *) pPtr);
+         break;
+
+        case INDI_TEXT:
+        delete ((ITextVectorProperty *) pPtr);
+        break;
+
+        case INDI_SWITCH:
+        delete ((ISwitchVectorProperty *) pPtr);
+        break;
+
+        case INDI_LIGHT:
+        delete ((ILightVectorProperty *) pPtr);
+        break;
+
+        case INDI_BLOB:
+        delete ((IBLOBVectorProperty *) pPtr);
+        break;
+
+    }
+  }
+}
+
+void PropertyContainer::setProperty(void *p)
+{
+    pRegistered = true;
+    pPtr = p;
+
+}
+
+void PropertyContainer::setType(INDI_TYPE t)
+{
+    pType = t;
+}
+
+void PropertyContainer::setRegistered(bool r)
+{
+   pRegistered = r;
+}
+
+void PropertyContainer::setDynamic(bool d)
+{
+   pDynamic = d;
+}
+
 INDI::BaseDriver::BaseDriver()
 {
     mediator = NULL;
     lp = newLilXML();
+    char indidev[MAXINDIDEVICE];
+    strncpy(indidev, "INDIDEV=", MAXINDIDEVICE);
 
     if (getenv("INDIDEV") != NULL)
     {
-        strncpy(deviceID, getenv("INDIDEV"), MAXINDINAME);
-        putenv("INDIDEV=");
+        strncpy(deviceID, getenv("INDIDEV"), MAXINDIDEVICE);
+        putenv(indidev);
     }
 }
 
@@ -24,13 +106,18 @@ INDI::BaseDriver::BaseDriver()
 INDI::BaseDriver::~BaseDriver()
 {
     delLilXML (lp);
+    while(!pAll.empty())
+    {
+      delete pAll.back();
+      pAll.pop_back();
+    }
 }
 
 INumberVectorProperty * INDI::BaseDriver::getNumber(const char *name)
 {
     INumberVectorProperty * nvp = NULL;
 
-    nvp = static_cast<INumberVectorProperty *> (getProperty(name, INDI_NUMBER));
+    nvp = static_cast<INumberVectorProperty *> (getProperty(name, PropertyContainer::INDI_NUMBER));
 
     return nvp;
 }
@@ -39,7 +126,7 @@ ITextVectorProperty * INDI::BaseDriver::getText(const char *name)
 {
     ITextVectorProperty * tvp = NULL;
 
-    tvp = static_cast<ITextVectorProperty *> (getProperty(name, INDI_TEXT));
+    tvp = static_cast<ITextVectorProperty *> (getProperty(name, PropertyContainer::INDI_TEXT));
 
     return tvp;
 }
@@ -48,7 +135,7 @@ ISwitchVectorProperty * INDI::BaseDriver::getSwitch(const char *name)
 {
     ISwitchVectorProperty * svp = NULL;
 
-    svp = static_cast<ISwitchVectorProperty *> (getProperty(name, INDI_SWITCH));
+    svp = static_cast<ISwitchVectorProperty *> (getProperty(name, PropertyContainer::INDI_SWITCH));
 
     return svp;
 }
@@ -57,7 +144,7 @@ ILightVectorProperty * INDI::BaseDriver::getLight(const char *name)
 {
     ILightVectorProperty * lvp = NULL;
 
-    lvp = static_cast<ILightVectorProperty *> (getProperty(name, INDI_LIGHT));
+    lvp = static_cast<ILightVectorProperty *> (getProperty(name, PropertyContainer::INDI_LIGHT));
 
     return lvp;
 }
@@ -66,14 +153,18 @@ IBLOBVectorProperty * INDI::BaseDriver::getBLOB(const char *name)
 {       
   IBLOBVectorProperty * bvp = NULL;
 
-  bvp = static_cast<IBLOBVectorProperty *> (getProperty(name, INDI_BLOB));
+  bvp = static_cast<IBLOBVectorProperty *> (getProperty(name, PropertyContainer::INDI_BLOB));
 
   return bvp;
 }
 
-void * INDI::BaseDriver::getProperty(const char *name, INDI_TYPE type)
+void * INDI::BaseDriver::getProperty(const char *name, PropertyContainer::INDI_TYPE type)
 {
-    std::map< boost::shared_ptr<void>, INDI_TYPE>::iterator orderi;
+    PropertyContainer::INDI_TYPE pType;
+    void *pPtr;
+    bool pRegistered = false;
+
+    std::vector<PropertyContainer *>::iterator orderi;
 
     INumberVectorProperty *nvp;
     ITextVectorProperty *tvp;
@@ -83,35 +174,128 @@ void * INDI::BaseDriver::getProperty(const char *name, INDI_TYPE type)
 
     for (orderi = pAll.begin(); orderi != pAll.end(); orderi++)
     {
-        if (type != INDI_UNKNOWN &&  orderi->second != type)
+        pType       = (*orderi)->getType();
+        pPtr        = (*orderi)->getProperty();
+        pRegistered = (*orderi)->getRegistered();
+
+        if (type != PropertyContainer::INDI_UNKNOWN && pType != type)
             continue;
 
-        switch (orderi->second)
+        switch (pType)
         {
-        case INDI_NUMBER:
-            nvp = static_cast<INumberVectorProperty *>((orderi->first).get());
-            if (!strcmp(name, nvp->name))
-                return (orderi->first).get();
+        case PropertyContainer::INDI_NUMBER:
+            nvp = static_cast<INumberVectorProperty *>(pPtr);
+            if (nvp == NULL)
+                continue;
+
+            if (!strcmp(name, nvp->name) && pRegistered)
+                return pPtr;
              break;
-        case INDI_TEXT:
-             tvp = static_cast<ITextVectorProperty *>((orderi->first).get());
-             if (!strcmp(name, tvp->name))
-                return (orderi->first).get();
+        case PropertyContainer::INDI_TEXT:
+             tvp = static_cast<ITextVectorProperty *>(pPtr);
+             if (tvp == NULL)
+                 continue;
+
+             if (!strcmp(name, tvp->name)  && pRegistered)
+                return pPtr;
              break;
-        case INDI_SWITCH:
-             svp = static_cast<ISwitchVectorProperty *>((orderi->first).get());
-             if (!strcmp(name, svp->name))
-                 return (orderi->first).get();
+        case PropertyContainer::INDI_SWITCH:
+             svp = static_cast<ISwitchVectorProperty *>(pPtr);
+             if (svp == NULL)
+                 continue;
+
+             //IDLog("Switch %s and aux value is now %d\n", svp->name, regStatus );
+             if (!strcmp(name, svp->name) && pRegistered)
+                 return pPtr;
              break;
-        case INDI_LIGHT:
-             lvp = static_cast<ILightVectorProperty *>((orderi->first).get());
-             if (!strcmp(name, lvp->name))
-                 return (orderi->first).get();
+        case PropertyContainer::INDI_LIGHT:
+             lvp = static_cast<ILightVectorProperty *>(pPtr);
+             if (lvp == NULL)
+                 continue;
+
+             if (!strcmp(name, lvp->name)  && pRegistered)
+                 return pPtr;
              break;
-        case INDI_BLOB:
-             bvp = static_cast<IBLOBVectorProperty *>((orderi->first).get());
-             if (!strcmp(name, bvp->name))
-                 return (orderi->first).get();
+        case PropertyContainer::INDI_BLOB:
+             bvp = static_cast<IBLOBVectorProperty *>(pPtr);
+             if (bvp == NULL)
+                 continue;
+
+             if (!strcmp(name, bvp->name) && pRegistered)
+                 return pPtr;
+             break;
+        }
+
+    }
+
+    return NULL;
+}
+
+PropertyContainer * INDI::BaseDriver::getContainer(const char *name, PropertyContainer::INDI_TYPE type)
+{
+    PropertyContainer::INDI_TYPE pType;
+    void *pPtr;
+    bool pRegistered = false;
+
+    std::vector<PropertyContainer *>::iterator orderi;
+
+    INumberVectorProperty *nvp;
+    ITextVectorProperty *tvp;
+    ISwitchVectorProperty *svp;
+    ILightVectorProperty *lvp;
+    IBLOBVectorProperty *bvp;
+
+    for (orderi = pAll.begin(); orderi != pAll.end(); orderi++)
+    {
+        pType       = (*orderi)->getType();
+        pPtr        = (*orderi)->getProperty();
+        pRegistered = (*orderi)->getRegistered();
+
+        if (type != PropertyContainer::INDI_UNKNOWN && pType != type)
+            continue;
+
+        switch (pType)
+        {
+        case PropertyContainer::INDI_NUMBER:
+            nvp = static_cast<INumberVectorProperty *>(pPtr);
+            if (nvp == NULL)
+                continue;
+
+            if (!strcmp(name, nvp->name) && pRegistered)
+                return *orderi;
+             break;
+        case PropertyContainer::INDI_TEXT:
+             tvp = static_cast<ITextVectorProperty *>(pPtr);
+             if (tvp == NULL)
+                 continue;
+
+             if (!strcmp(name, tvp->name)  && pRegistered)
+                return *orderi;
+             break;
+        case PropertyContainer::INDI_SWITCH:
+             svp = static_cast<ISwitchVectorProperty *>(pPtr);
+             if (svp == NULL)
+                 continue;
+
+             //IDLog("Switch %s and aux value is now %d\n", svp->name, regStatus );
+             if (!strcmp(name, svp->name) && pRegistered)
+                 return *orderi;
+             break;
+        case PropertyContainer::INDI_LIGHT:
+             lvp = static_cast<ILightVectorProperty *>(pPtr);
+             if (lvp == NULL)
+                 continue;
+
+             if (!strcmp(name, lvp->name)  && pRegistered)
+                 return *orderi;
+             break;
+        case PropertyContainer::INDI_BLOB:
+             bvp = static_cast<IBLOBVectorProperty *>(pPtr);
+             if (bvp == NULL)
+                 continue;
+
+             if (!strcmp(name, bvp->name) && pRegistered)
+                 return *orderi;
              break;
         }
 
@@ -121,8 +305,11 @@ void * INDI::BaseDriver::getProperty(const char *name, INDI_TYPE type)
 }
 
 int INDI::BaseDriver::removeProperty(const char *name)
-{
-    std::map< boost::shared_ptr<void>, INDI_TYPE>::iterator orderi;
+{    
+    std::vector<PropertyContainer *>::iterator orderi;
+
+    PropertyContainer::INDI_TYPE pType;
+    void *pPtr;
 
     INumberVectorProperty *nvp;
     ITextVectorProperty *tvp;
@@ -132,45 +319,48 @@ int INDI::BaseDriver::removeProperty(const char *name)
 
     for (orderi = pAll.begin(); orderi != pAll.end(); orderi++)
     {
-        switch (orderi->second)
+        pType       = (*orderi)->getType();
+        pPtr        = (*orderi)->getProperty();
+
+        switch (pType)
         {
-        case INDI_NUMBER:
-            nvp = static_cast<INumberVectorProperty *>((orderi->first).get());
+        case PropertyContainer::INDI_NUMBER:
+            nvp = static_cast<INumberVectorProperty *>(pPtr);
             if (!strcmp(name, nvp->name))
             {
-                 pAll.erase(orderi);
+                (*orderi)->setRegistered(false);
                  return 0;
              }
              break;
-        case INDI_TEXT:
-             tvp = static_cast<ITextVectorProperty *>((orderi->first).get());
+        case PropertyContainer::INDI_TEXT:
+             tvp = static_cast<ITextVectorProperty *>(pPtr);
              if (!strcmp(name, tvp->name))
              {
-                  pAll.erase(orderi);
+                  (*orderi)->setRegistered(false);
                   return 0;
               }
              break;
-        case INDI_SWITCH:
-             svp = static_cast<ISwitchVectorProperty *>((orderi->first).get());
+        case PropertyContainer::INDI_SWITCH:
+             svp = static_cast<ISwitchVectorProperty *>(pPtr);
              if (!strcmp(name, svp->name))
              {
-                  pAll.erase(orderi);
+                 (*orderi)->setRegistered(false);
                   return 0;
               }
              break;
-        case INDI_LIGHT:
-             lvp = static_cast<ILightVectorProperty *>((orderi->first).get());
+        case PropertyContainer::INDI_LIGHT:
+             lvp = static_cast<ILightVectorProperty *>(pPtr);
              if (!strcmp(name, lvp->name))
              {
-                  pAll.erase(orderi);
+                 (*orderi)->setRegistered(false);
                   return 0;
               }
              break;
-        case INDI_BLOB:
-             bvp = static_cast<IBLOBVectorProperty *>((orderi->first).get());
+        case PropertyContainer::INDI_BLOB:
+             bvp = static_cast<IBLOBVectorProperty *>(pPtr);
              if (!strcmp(name, bvp->name))
              {
-                  pAll.erase(orderi);
+                 (*orderi)->setRegistered(false);
                   return 0;
               }
              break;
@@ -214,10 +404,8 @@ int INDI::BaseDriver::buildProp(XMLEle *root, char *errmsg)
 {
     IPerm perm;
     IPState state;
-    ISRule rule;
     XMLEle *ep = NULL;
     char *rtag, *rname, *rdev;
-    //INDI_TYPE type;
     double timeout=0;
 
     rtag = tagXMLEle(root);
@@ -257,10 +445,9 @@ int INDI::BaseDriver::buildProp(XMLEle *root, char *errmsg)
 
     if (!strcmp (rtag, "defNumberVector"))
     {
+        PropertyContainer *indiProp = new PropertyContainer();
+        INumberVectorProperty *nvp = new INumberVectorProperty;
 
-        numberPtr nvp(new INumberVectorProperty);
-        //INumberVectorProperty *nvp = new INumberVectorProperty;
-        //INumberVectorProperty *nvp = (INumberVectorProperty *) malloc(sizeof(INumberVectorProperty));
         INumber *np = NULL;
         int n=0;
 
@@ -280,7 +467,7 @@ int INDI::BaseDriver::buildProp(XMLEle *root, char *errmsg)
         {
             np = (INumber *) realloc(np, (n+1) * sizeof(INumber));
 
-            np[n].nvp = nvp.get();
+            np[n].nvp = nvp;
 
             XMLAtt *na = findXMLAtt (ep, "name");
 
@@ -319,11 +506,13 @@ int INDI::BaseDriver::buildProp(XMLEle *root, char *errmsg)
     {
         nvp->nnp = n;
         nvp->np  = np;
-        //orderPtr o(new pOrder);
-        //o->p = &nvp;
-        //o->type = INDI_NUMBER;
-        //pAll.push_back(o);
-        pAll[nvp] = INDI_NUMBER;
+
+        indiProp->setProperty(nvp);
+        indiProp->setDynamic(true);
+        indiProp->setType(PropertyContainer::INDI_NUMBER);
+
+        pAll.push_back(indiProp);
+
         //IDLog("Adding number property %s to list.\n", nvp->name);
         if (mediator)
             mediator->newProperty(nvp->device, nvp->name);
@@ -333,9 +522,9 @@ int INDI::BaseDriver::buildProp(XMLEle *root, char *errmsg)
 }
   else if (!strcmp (rtag, "defSwitchVector"))
         {
-            switchPtr svp(new ISwitchVectorProperty);
-            //ISwitchVectorProperty *svp = new ISwitchVectorProperty;
-            //ISwitchVectorProperty *svp = (ISwitchVectorProperty *) malloc(sizeof(ISwitchVectorProperty));
+            PropertyContainer *indiProp = new PropertyContainer();
+            ISwitchVectorProperty *svp = new ISwitchVectorProperty;
+
             ISwitch *sp = NULL;
             int n=0;
 
@@ -359,7 +548,7 @@ int INDI::BaseDriver::buildProp(XMLEle *root, char *errmsg)
             {
                 sp = (ISwitch *) realloc(sp, (n+1) * sizeof(ISwitch));
 
-                sp[n].svp = svp.get();
+                sp[n].svp = svp;
 
                 XMLAtt *na = findXMLAtt (ep, "name");
 
@@ -379,11 +568,12 @@ int INDI::BaseDriver::buildProp(XMLEle *root, char *errmsg)
         {
             svp->nsp = n;
             svp->sp  = sp;
-            //orderPtr o(new pOrder);
-            //o->p = &svp;
-            //o->type = INDI_SWITCH;
-            //pAll.push_back(o);
-            pAll[svp] = INDI_SWITCH;
+
+            indiProp->setProperty(svp);
+            indiProp->setDynamic(true);
+            indiProp->setType(PropertyContainer::INDI_SWITCH);
+
+            pAll.push_back(indiProp);
             //IDLog("Adding Switch property %s to list.\n", svp->name);
             if (mediator)
                 mediator->newProperty(svp->device, svp->name);
@@ -395,9 +585,8 @@ int INDI::BaseDriver::buildProp(XMLEle *root, char *errmsg)
 else if (!strcmp (rtag, "defTextVector"))
     {
 
-        //ITextVectorProperty *tvp = new ITextVectorProperty;
-        //ITextVectorProperty *tvp = (ITextVectorProperty *) malloc(sizeof(ITextVectorProperty));
-        textPtr tvp(new ITextVectorProperty);
+        PropertyContainer *indiProp = new PropertyContainer();
+        ITextVectorProperty *tvp = new ITextVectorProperty;
         IText *tp = NULL;
         int n=0;
 
@@ -417,7 +606,7 @@ else if (!strcmp (rtag, "defTextVector"))
         {
             tp = (IText *) realloc(tp, (n+1) * sizeof(IText));
 
-            tp[n].tvp = tvp.get();
+            tp[n].tvp = tvp;
 
             XMLAtt *na = findXMLAtt (ep, "name");
 
@@ -438,11 +627,12 @@ else if (!strcmp (rtag, "defTextVector"))
     {
         tvp->ntp = n;
         tvp->tp  = tp;
-        //orderPtr o(new pOrder);
-        //o->p = &tvp;
-        //o->type = INDI_TEXT;
-        //pAll.push_back(o);
-        pAll[tvp] = INDI_TEXT;
+
+        indiProp->setProperty(tvp);
+        indiProp->setDynamic(true);
+        indiProp->setType(PropertyContainer::INDI_TEXT);
+
+        pAll.push_back(indiProp);
         //IDLog("Adding Text property %s to list.\n", tvp->name);
         if (mediator)
             mediator->newProperty(tvp->device, tvp->name);
@@ -453,9 +643,8 @@ else if (!strcmp (rtag, "defTextVector"))
 else if (!strcmp (rtag, "defLightVector"))
     {
 
-        //ILightVectorProperty *lvp = new ILightVectorProperty;
-        //ILightVectorProperty *lvp = (ILightVectorProperty *) malloc(sizeof(ILightVectorProperty));
-        lightPtr lvp(new ILightVectorProperty);
+        PropertyContainer *indiProp = new PropertyContainer();
+        ILightVectorProperty *lvp = new ILightVectorProperty;
         ILight *lp = NULL;
         int n=0;
 
@@ -473,7 +662,7 @@ else if (!strcmp (rtag, "defLightVector"))
         {
             lp = (ILight *) realloc(lp, (n+1) * sizeof(ILight));
 
-            lp[n].lvp = lvp.get();
+            lp[n].lvp = lvp;
 
             XMLAtt *na = findXMLAtt (ep, "name");
 
@@ -494,11 +683,13 @@ else if (!strcmp (rtag, "defLightVector"))
     {
         lvp->nlp = n;
         lvp->lp  = lp;
-        //orderPtr o(new pOrder);
-        //o->p = &lvp;
-        //o->type = INDI_LIGHT;
-        //pAll.push_back(o);
-        pAll[lvp] = INDI_LIGHT;
+
+        indiProp->setProperty(lvp);
+        indiProp->setDynamic(true);
+        indiProp->setType(PropertyContainer::INDI_LIGHT);
+
+        pAll.push_back(indiProp);
+
         //IDLog("Adding Light property %s to list.\n", lvp->name);
         if (mediator)
             mediator->newProperty(lvp->device, lvp->name);
@@ -508,10 +699,8 @@ else if (!strcmp (rtag, "defLightVector"))
 }
 else if (!strcmp (rtag, "defBLOBVector"))
     {
-
-        //IBLOBVectorProperty *bvp = new IBLOBVectorProperty;
-        //IBLOBVectorProperty *bvp = (IBLOBVectorProperty *) malloc(sizeof(IBLOBVectorProperty));
-        blobPtr bvp(new IBLOBVectorProperty);
+        PropertyContainer *indiProp = new PropertyContainer();
+        IBLOBVectorProperty *bvp = new IBLOBVectorProperty;
         IBLOB *bp = NULL;
         int n=0;
 
@@ -529,7 +718,7 @@ else if (!strcmp (rtag, "defBLOBVector"))
         {
             bp = (IBLOB *) realloc(bp, (n+1) * sizeof(IBLOB));
 
-            bp[n].bvp = bvp.get();
+            bp[n].bvp = bvp;
 
             XMLAtt *na = findXMLAtt (ep, "name");
 
@@ -559,11 +748,12 @@ else if (!strcmp (rtag, "defBLOBVector"))
     {
         bvp->nbp = n;
         bvp->bp  = bp;
-        //orderPtr o(new pOrder);
-        //o->p = &bvp;
-        //o->type = INDI_BLOB;
-        //pAll.push_back(o);
-        pAll[bvp] = INDI_BLOB;
+
+        indiProp->setProperty(bvp);
+        indiProp->setDynamic(true);
+        indiProp->setType(PropertyContainer::INDI_BLOB);
+
+        pAll.push_back(indiProp);
         //IDLog("Adding BLOB property %s to list.\n", bvp->name);
         if (mediator)
             mediator->newProperty(bvp->device, bvp->name);
@@ -912,64 +1102,93 @@ void INDI::BaseDriver::addMessage(const char *msg)
     messageQueue.append(msg);
 }
 
-void INDI::BaseDriver::registerProperty(void *p, INDI_TYPE type)
+void INDI::BaseDriver::registerProperty(void *p, PropertyContainer::INDI_TYPE type)
 {
+    PropertyContainer *pContainer;
 
-
-    if (type == INDI_NUMBER)
+    if (type == PropertyContainer::INDI_NUMBER)
     {
         INumberVectorProperty *nvp = static_cast<INumberVectorProperty *> (p);
-        if (getProperty(nvp->name, INDI_NUMBER) != NULL)
+        if ( (pContainer = getContainer(nvp->name, PropertyContainer::INDI_NUMBER)) != NULL)
+        {
+            pContainer->setRegistered(true);
             return;
+        }
 
-        numberPtr ovp(nvp);
+        pContainer = new PropertyContainer();
+        pContainer->setProperty(p);
+        pContainer->setType(type);
+
+        pAll.push_back(pContainer);
 
-        pAll[ovp] = INDI_NUMBER;
     }
-    else if (type == INDI_TEXT)
+    else if (type == PropertyContainer::INDI_TEXT)
     {
        ITextVectorProperty *tvp = static_cast<ITextVectorProperty *> (p);
-       if (getProperty(tvp->name, INDI_TEXT) != NULL)
+
+       if ( (pContainer = getContainer(tvp->name, PropertyContainer::INDI_TEXT)) != NULL)
+       {
+           pContainer->setRegistered(true);
            return;
+       }
+
+       pContainer = new PropertyContainer();
+       pContainer->setProperty(p);
+       pContainer->setType(type);
+
+       pAll.push_back(pContainer);
 
-       textPtr ovp(tvp);
-       //o->p = &ovp;
 
-       pAll[ovp] = INDI_TEXT;
    }
-    else if (type == INDI_SWITCH)
+    else if (type == PropertyContainer::INDI_SWITCH)
     {
        ISwitchVectorProperty *svp = static_cast<ISwitchVectorProperty *> (p);
-       if (getProperty(svp->name, INDI_SWITCH) != NULL)
+
+       if ( (pContainer = getContainer(svp->name, PropertyContainer::INDI_SWITCH)) != NULL)
+       {
+           pContainer->setRegistered(true);
            return;
+       }
 
-       switchPtr ovp(svp);
-       //o->p = &ovp;
+       pContainer = new PropertyContainer();
+       pContainer->setProperty(p);
+       pContainer->setType(type);
 
-       IDLog("Registering switch %s\n", svp->name);
+       pAll.push_back(pContainer);
 
-       pAll[ovp] = INDI_SWITCH;
     }
-    else if (type == INDI_LIGHT)
+    else if (type == PropertyContainer::INDI_LIGHT)
     {
        ILightVectorProperty *lvp = static_cast<ILightVectorProperty *> (p);
-       if (getProperty(lvp->name, INDI_LIGHT) != NULL)
-           return;
 
-       lightPtr ovp(lvp);
-       //o->p = &ovp;
-       pAll[ovp] = INDI_LIGHT;
+       if ( (pContainer = getContainer(lvp->name, PropertyContainer::INDI_LIGHT)) != NULL)
+       {
+           pContainer->setRegistered(true);
+           return;
+       }
+
+       pContainer = new PropertyContainer();
+       pContainer->setProperty(p);
+       pContainer->setType(type);
+
+       pAll.push_back(pContainer);
    }
-    else if (type == INDI_BLOB)
+    else if (type == PropertyContainer::INDI_BLOB)
     {
        IBLOBVectorProperty *bvp = static_cast<IBLOBVectorProperty *> (p);
-       if (getProperty(bvp->name, INDI_BLOB) != NULL)
+
+       if ( (pContainer = getContainer(bvp->name, PropertyContainer::INDI_BLOB)) != NULL)
+       {
+           pContainer->setRegistered(true);
            return;
+       }
 
-       blobPtr ovp(bvp);
-       //o->p = &ovp;
+       pContainer = new PropertyContainer();
+       pContainer->setProperty(p);
+       pContainer->setType(type);
+
+       pAll.push_back(pContainer);
 
-       pAll[ovp] = INDI_BLOB;
     }
 
 }
